# $Copyright: Copyright (c) 2017 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

package OpenStackIP;

@ISA = ('Exporter');
@EXPORT = ('ValidateCommandPath','getTokenAndNeutronEndPoint','AcquireSharedLock','getMacAddress','getFloatingIpId','deassociateFloatingIP','associateFloatingIP','getFixedIPAssociatedFloatingIP','isFixedIPPresentOnPort','getMacPortAndNetworkId','getAllFixedIPsOnPort','addFixedIPsOnPort','getPortIdForAddressInFixedIP','AcquireLock','ReleaseLock','getAllAllowedAddressPairOnPort','addAllowedAddressPairOnPort','isAllowedAddressPairPresentOnPort','getPortIdForAddressInAllowedAddressPair','getRouterId','getAllRoutesOfRouter','addStaticRoutesOnRouter','isStaticRoutePresentOnRouter','isCIDRPresentOnRouter','validateIpaddress','IsIpPartOfSubnet','updateRouter','createPort','deletePort');

use strict;
use JSON;
use Fcntl qw(:flock);
use POSIX;
use ag_i18n_inc;
use vcs_net_utils_inc;

#Attributes and Keywords Name
my ($pPrivateIP,$pFloatingIP,$pBaseIP,$pAllowedAddressPair,$pOverlayIP,$pRouteTables,$kwCidr,$pUseNewPort,$kwNextHop) = ("PrivateIP","FloatingIP","BaseIP","AllowedAddressPair","OverlayIP","RouteTables","CIDR","UseNewPort","NextHop");

#Set and Validate commands path.
my $curlPath = "/usr/bin/curl";
my $tailPath = "/usr/bin/tail";
my $catPath = "/usr/bin/cat";

sub ValidateCommandPath {
        my $cmd=shift;
        my $cmdPath = "";

        if(-e "/usr/bin/$cmd") {
                $cmdPath = "/usr/bin/$cmd";
        }
        elsif( -e "/bin/$cmd") {
                $cmdPath = "/bin/$cmd";
        }
        elsif( -e "/usr/local/bin/$cmd") {
                $cmdPath = "/usr/local/bin/$cmd";
        }
        else {
		return "";
        }
        return $cmdPath;
}

$curlPath = ValidateCommandPath('curl');
$tailPath = ValidateCommandPath('tail');
$catPath = ValidateCommandPath('cat');


#Read the OpenStack Auth file and get neutron admin URL,Token & TenantId.
#This sub routine called at the start of each entry point.
sub getTokenAndNeutronEndPoint {

	my $auth_file ="$VCSLOG/lock/.OpenStackAuth";
	my $auth_lockfile = "$VCSLOG/lock/.OpenStackAuth.lck";

	if(! -e $auth_file) {
		return;	
	}

	#Acquiring the shared lock.
	my $auth_lockfilehandle = AcquireSharedLock($auth_lockfile);
	open(my $auth_filehandle,"$auth_file");
        my $readInfo = <$auth_filehandle>;
	close($auth_filehandle);
	ReleaseLock($auth_lockfilehandle);

	my @readInfoArray=split(":;",$readInfo);
        my $token=@readInfoArray['0'];
        $token = get_decrypt_token($token);
	my $tenantid=@readInfoArray['1'];
	my $neutronAdminUrl=@readInfoArray['2'];
	return ($token,$neutronAdminUrl,$tenantid);
}

#Sub routine to acquire the shared lock.
sub AcquireSharedLock
{
        my ($FileName) = @_;
        my $flock_struct = "s s l l i";
        my $buf = pack($flock_struct, F_RDLCK, SEEK_SET, 0, 0, 0);

        open(FILE, $FileName) or die "could not open file";
        $| = 1;
        fcntl(FILE, F_SETLKW, $buf) or die "fcntl(F_SETLK): $!";
        return *FILE;
}



# Get Mac Address of the Device.
sub getMacAddress {
        my $device = shift;
        my $macAddress = `$catPath /sys/class/net/$device/address`;
        chomp $macAddress;
        return($macAddress);
}


# Get the ID of the FloatingIP.
sub getFloatingIpId {

	my $token = shift;
	my $neutronAdminUrl = shift;
	my $floatingIP = shift;
	my $tenantId = shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/floatingips.json?tenant_id=$tenantId -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1`);

	my @floatingIps = @{ $tempOutput->{'floatingips'} };
        foreach my $eachIP ( @floatingIps ) {

		my $ip=$eachIP->{'floating_ip_address'};
		my $id=$eachIP->{'id'};
		if($ip eq $floatingIP) {
			return $id;
		}
   	}
 	return;
}

#Dissociate FloatingIP from the Device Port.
sub deassociateFloatingIP {

	my $token = shift;
	my $neutronAdminUrl = shift;
	my $floatingIpId= shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X PUT $neutronAdminUrl/floatingips/$floatingIpId.json -H "User-Agent: python-neutronclient" -H "Content-Type: application/json" -H "Accept: application/json" -H "X-Auth-Token:$token" -d '{"floatingip": {"port_id": null}}' | $tailPath -n 1 `);

        if(exists($tempOutput->{'NeutronError'})) {
	        my $errorMsg = $tempOutput->{'NeutronError'}{'message'};
                VCSAG_LOG_MSG("E", "Failed to dissociate $pFloatingIP due to error $errorMsg",20030,"$pFloatingIP","$errorMsg");
                return 0;
        }
	return 1;
}


#Associate FloatingIP to the Device Port.
sub associateFloatingIP {
	
	my $token = shift;
	my $neutronAdminUrl = shift;
	my $PrivateIP = shift;
	my $floatingIpId = shift;
	my $PortId = shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X PUT $neutronAdminUrl/floatingips/$floatingIpId.json -H "User-Agent: python-neutronclient" -H "Content-Type:application/json" -H "Accept: application/json" -H "X-Auth-Token:$token" -d '{"floatingip": {"fixed_ip_address":"$PrivateIP", "port_id":"$PortId"}}' | $tailPath -n 1 `);

	if(exists($tempOutput->{'NeutronError'})) {  
		my $errorMsg = $tempOutput->{'NeutronError'}{'message'};
		VCSAG_LOG_MSG("E", "Failed to associate $pFloatingIP due to error $errorMsg",20029,"$pFloatingIP","$errorMsg");
		return 0;
	}
	return 1;

}


#Get PrivateIP(fixed_ip field IP) to which given FloatingIP associated.
sub getFixedIPAssociatedFloatingIP {
 
	my $token = shift;
	my $neutronAdminUrl = shift;
	my $floatingIpId = shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/floatingips/$floatingIpId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" |  $tailPath -n 1 `);

	my $fixedIp=$tempOutput->{'floatingip'}{'fixed_ip_address'};
	return $fixedIp;
	
}



#Check if PrivateIP is present on the Device Port.
#return 1 : PrivateIP found on the Device Port 
#return 0 : PrivateIP not found on Device Port
sub isFixedIPPresentOnPort {

	my $token = shift;
        my $neutronAdminUrl = shift;
        my $portId = shift;
        my $privateIP = shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1`);

	my @fixedIPs=@{ $tempOutput->{'port'}{'fixed_ips'}};
        foreach my $eachIP (@fixedIPs) {
           my $fixedIP=$eachIP->{'ip_address'};
            if($privateIP eq $fixedIP) {
              	return 1;
            	}
	}	
	return 0;
}


#Get PortID and NetworkID of the provided mac address.
#return PortID and NetworkID of the Device.
sub getMacPortAndNetworkId {
	my $token = shift;
	my $neutronAdminUrl = shift;
	my $macAddress = shift;
	my $tenantId = shift;

        my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports.json?tenant_id=$tenantId -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1 `);

        my @ports = @ { $tempOutput->{'ports'} };
        foreach my $port (@ports) {
               	my $portId=$port->{'id'};
		my $networkId=$port->{'network_id'};
               	my $mac=$port->{'mac_address'};
               	if($mac eq $macAddress) {
			return ($portId,$networkId);
		}
	}
	return;	
}


#Get all the PrivateIPs (fixed_ips) present on the port.
#return : Array of PrivateIPs
sub getAllFixedIPsOnPort {
	my $token = shift;
        my $neutronAdminUrl = shift;
	my $portId = shift;
	my @allFixedIps;
	
	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1 `);

	my @pairs=@{ $tempOutput->{'port'}{'fixed_ips'} };
	foreach my $pair (@pairs) {
		my $ipAddress = $pair->{'ip_address'};
		push @allFixedIps,$ipAddress;
	}
	return @allFixedIps;
}


#Add PrivateIPs to the port.
sub addFixedIPsOnPort{

	my $token = shift;
	my $neutronAdminUrl = shift;
	my $portId = shift;
	my $allFixedIps =shift;

	my $strips= "";
	foreach my $ip (@$allFixedIps) {
		$strips=$strips. "{\"ip_address\": \"".$ip."\"},";
	}
	chop $strips;
	my $tempOutput =  from_json(`$curlPath -s -g -i -X PUT $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Content-Type: application/json" -H "Accept: application/json" -H "X-Auth-Token:$token" -d '{"port": {"fixed_ips": [$strips]}}' | $tailPath -n 1 `);

        if(exists($tempOutput->{'NeutronError'})) {
                my $errorMsg = $tempOutput->{'NeutronError'}{'message'};
                VCSAG_LOG_MSG("E", "Failed to add $pPrivateIP to the neutron port due to error $errorMsg",20031,"$pPrivateIP","$errorMsg");
                return 0;
        }

	return 1;
}

#Get port id of the the private ip(fixed ip).
#return PortID : If given address(PrivateIP) is present on any port.
#return	null : Not present.
sub getPortIdForAddressInFixedIP {

        my $token = shift;
        my $neutronAdminUrl = shift;
	my $fixedIp = shift;
	
	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports.json?fixed_ips=ip_address%3D$fixedIp -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1 `);
	
	my @ports= @{ $tempOutput->{'ports'} };

	if(@ports[0] ne ""){
		my $id = @ports[0]->{'id'};
		my $portTag = @ports[0]->{'name'};
		return ($id,$portTag);
	}

	return;
}

#Acquire exclusive Lock on the file before any port updation.
sub AcquireLock
{
        my ($lock_file) = (@_);
        local *LHANDLE;
        my ($arg, $ret_val) = ("", 0);

        open (LHANDLE, ">$lock_file");
        $arg = pack ("SSLLLLLLLL", F_WRLCK (), 0, 0, 0, 0, 0, 0, 0, 0, 0);
        $ret_val = fcntl(LHANDLE, F_SETLKW (), $arg);

        if ($ret_val < 0) {
                VCSAG_LOG_MSG ("E", "Could not get lock on file $lock_file", 20028, "$lock_file");
                exit 0;
        }

        return *LHANDLE;
}

#Release lock from the file after port updation.
sub ReleaseLock
{
        close ($_[0]);
}



#Get all IPs present on the "allowed_address_pair" field of the port.
#return array of IPs/CIDR
sub getAllAllowedAddressPairOnPort {
        my $token = shift;
        my $neutronAdminUrl = shift;
        my $portId = shift;
        my @allAddress;

        my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1 `);

        my @pairs=@{ $tempOutput->{'port'}{'allowed_address_pairs'} };

        foreach my $pair (@pairs) {
                my $ipAddress = $pair->{'ip_address'};
                push @allAddress,$ipAddress;
        }

        return @allAddress;

}


#Add OverlayIP or PrivateIP (UseNewPort = 1) at the allowed_address_pair field of the port.
sub addAllowedAddressPairOnPort{

        my $token = shift;
        my $neutronAdminUrl = shift;
        my $portId = shift;
	my $allAddress = shift;
 
        my $strips= "";
        foreach my $ip (@$allAddress)
        {
                $strips=$strips. "{\"ip_address\": \"".$ip."\"},";
        }
        chop $strips;
        my $tempOutput = from_json(`$curlPath -s -g -i -X PUT $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Content-Type: application/json" -H "Accept: application/json" -H "X-Auth-Token:$token" -d '{"port": {"allowed_address_pairs": [$strips]}}' | $tailPath -n 1 `);

        if(exists($tempOutput->{'NeutronError'})) {
                my $errorMsg = $tempOutput->{'NeutronError'}{'message'};
                VCSAG_LOG_MSG("E", "Failed to add IP to the neutron port due to error $errorMsg",20032,"$errorMsg");
                return 0;
        }

	return 1;
}


#Check OverlayIP or PrivateIP (UseNewPort = 1) present on the port allowed_address_pair field.
#return 1 : IP present on the port
#return 0 : IP not present on the port
sub isAllowedAddressPairPresentOnPort {

        my $token = shift;
        my $neutronAdminUrl = shift;
        my $portId = shift;
        my $address = shift;

        my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1 `);

        my @pairs=@{ $tempOutput->{'port'}{'allowed_address_pairs'} };

        foreach my $pair (@pairs) {
                my $ipAddress = $pair->{'ip_address'};

		if($ipAddress eq $address) {
			return 1;
		}
        }

        return 0;
}


sub getPortIdForAddressInAllowedAddressPair{

        my $token = shift;
        my $neutronAdminUrl = shift;
        my $address = shift;
	my $tenantId = shift;

        my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/ports.json?tenant_id=$tenantId -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1 `);

        my @ports = @ { $tempOutput->{'ports'} };
        foreach my $port (@ports) {
                my @pairs=@{ $port->{'allowed_address_pairs'} };
                my $portId = $port->{'id'};
                foreach my $pair (@pairs) {
                        my $ipAddress = $pair->{'ip_address'};
                        if($ipAddress eq $address) {
                                return $portId;
                        }
                 }
        }

        return;

}

#Get the ID of the router.
#return RouterID else return null (empty)
sub getRouterId {
	my $token = shift;
	my $neutronAdminUrl = shift;
	my $routerName = shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/routers.json?name=$routerName -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1`);

	my @allRouters = @{ $tempOutput->{'routers'} };
	foreach my $router (@allRouters) {
		my $rtId = $router->{'id'};
		my $rtName = $router->{'name'};
		if($rtName eq $routerName) {
			return $rtId;	
		}
	}
	return;
}

#Get All the static routes of the router.
#return array of routes
sub getAllRoutesOfRouter {
        my $token = shift;
        my $neutronAdminUrl = shift;
        my $routerId = shift;
        my @allRoutes;
	my $tempStr="";

        my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/routers/$routerId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1`);

        my @pairs=@{ $tempOutput->{'router'}{'routes'} };

        foreach my $pair (@pairs) {
                my $destination = $pair->{'destination'};
		my $nextHop = $pair->{'nexthop'};
		$tempStr=$destination ."-".$nextHop;		
                push @allRoutes,$tempStr;
        }

        return @allRoutes;

}


#Add Static routes to the router.
sub addStaticRoutesOnRouter {

        my $token = shift;
        my $neutronAdminUrl = shift;
        my $routerId = shift;
        my $allRoutes =shift;

        my $strips= "";
        foreach my $route (@$allRoutes) {
		my @eachRouteField = split("-",$route);
		my $destination = @eachRouteField[0];
		my $nextHop = @eachRouteField[1];

                $strips=$strips. "{\"destination\": \"".$destination."\",\"nexthop\": \"".$nextHop."\"},";
        }
        chop $strips;

        my $tempOutput = from_json(`$curlPath -s -g -i -X PUT $neutronAdminUrl/routers/$routerId.json -H "User-Agent: python-neutronclient" -H "Content-Type: application/json" -H "Accept: application/json" -H "X-Auth-Token:$token" -d '{"router": {"routes": [$strips]}}' | $tailPath -n 1`);

        if(exists($tempOutput->{'NeutronError'})) {
                my $errorMsg = $tempOutput->{'NeutronError'}{'message'};
                VCSAG_LOG_MSG("E", "Failed to add static route on the router due to error $errorMsg",20033,"$errorMsg");
                exit(0);
        }


}

#Check route entry present on the router.
#return 1 : route entry found on the router
#return 0 : route entry not found on the router
sub isStaticRoutePresentOnRouter {

        my $token = shift;
        my $neutronAdminUrl = shift;
        my $routerId = shift;
        my $route = shift;
	
	my @routeDetails = split("-",$route);
	my $routeDest=@routeDetails[0];
	my $routeNextHop=@routeDetails[1];

        my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/routers/$routerId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1`);

        my @pairs=@{ $tempOutput->{'router'}{'routes'} };

        foreach my $eachPair (@pairs) {
                my $destination = $eachPair->{'destination'};
		my $nextHop = $eachPair->{'nexthop'};
                if(($destination eq $routeDest) && ($nextHop eq $routeNextHop)) {
                        return 1;
                }
        }

        return 0;
}


#Check OverlayIP static route present on the router.
#return 1 : OverlayIP present on the router
#return 0 : Not Present
sub isCIDRPresentOnRouter {

	my $token = shift;
	my $neutronAdminUrl = shift;
	my $routerId = shift;
	my $cidr = shift;

	my $tempOutput = from_json(`$curlPath -s -g -i -X GET $neutronAdminUrl/routers/$routerId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token" | $tailPath -n 1`);

        my @pairs=@{ $tempOutput->{'router'}{'routes'} };

        foreach my $eachPair (@pairs) {
                my $destination = $eachPair->{'destination'};
		
		if($destination eq $cidr) {
			return 1;
		}
	}
	return 0;
}

# Validate all the given IPs format (IPv4).
# return 1 : all the provided IPs are in valid IPv4 format
# return 0 : Invalid IP
sub validateIpaddress {
	my $privateIP = shift;
	my $baseIP = shift;
	my $allowedAddress = shift;
	my $overlayIP = shift;
	my $floatingIP = shift;

	if((length($baseIP) !=0) && (VCSNetGetIPVersion($baseIP) == -1 )) {
		VCSAG_LOG_MSG("E", "Invalid $pBaseIP=$baseIP,IP not in proper IPv4 or IPv6 format",20034,"$pBaseIP","$baseIP");
		return(0);
	}
	if((length($floatingIP) !=0) && (VCSNetGetIPVersion($floatingIP) == -1 )) {
               VCSAG_LOG_MSG("E", "Invalid $pFloatingIP=$floatingIP,IP not in proper IPv4 or IPv6 format",20035,"$pFloatingIP","$floatingIP"); 
               return(0);
        }

	if((length($privateIP) !=0) && (VCSNetGetIPVersion($privateIP) == -1 )) {
               VCSAG_LOG_MSG("E", "Invalid $pPrivateIP=$privateIP,IP not in proper IPv4 or IPv6 format",20036,"$pPrivateIP","$privateIP");
               return(0);
        }

	if(length($allowedAddress) !=0) {
		my ($allowedIp,$allowedCidr) = split /\//, $allowedAddress;
		my $ipver = VCSNetGetIPVersion($allowedIp);
		if ($ipver == -1 ) {
			VCSAG_LOG_MSG("E", "Invalid $pAllowedAddressPair=$allowedAddress,$allowedIp address not in proper IPv4 or IPv6 format",20037,"$pAllowedAddressPair","$allowedAddress","$allowedIp"); 
			return(0);
        	}
		my $maxPrefixlen = ($ipver eq 4) ? 32 : 128;
		if (($allowedCidr < 0) || ($allowedCidr > $maxPrefixlen)) {
                	VCSAG_LOG_MSG("E", "Invalid $pAllowedAddressPair=$allowedAddress,$kwCidr $allowedCidr out of range",20038,"$pAllowedAddressPair","$allowedAddress","$kwCidr","$allowedCidr");
                	return(0);
        	}
	}

	#Valid OverlayIP example 10.100.10.10/32
	if(length($overlayIP) !=0) {
		my ($overlay_Ip,$overlayCidr) = split /\//, $overlayIP;
		my $ipver = VCSNetGetIPVersion($overlay_Ip);
		if($ipver == -1) {
			VCSAG_LOG_MSG("E", "Invalid $pOverlayIP=$overlayIP,$overlay_Ip address not in proper IPv4 or IPv6 format",20039,"$pOverlayIP","$overlayIP","$overlay_Ip");
			return(0);
		}
		my $maxPrefixlen = ($ipver eq 4) ? 32 : 128;
        	if (($overlayCidr < 0) || ($overlayCidr > $maxPrefixlen)) {
                	VCSAG_LOG_MSG("E", "Invalid $pOverlayIP=$overlayIP,$kwCidr $overlayCidr out of range",20040,"$pOverlayIP","$overlayIP","$kwCidr","$overlayCidr");
                	return(0);
        	}
	}
	return(1);
}


# Create a neutron port on the given tenant and network.
sub createPort {

        my $token = shift;
        my $neutronAdminUrl = shift;
        my $tenantId = shift;
        my $networkId =shift;
	my $privateIP = shift;
	my $portTag = shift;

        my $tempOutput =  from_json(`$curlPath -s -g -i -X POST $neutronAdminUrl/ports.json -H "User-Agent: python-neutronclient" -H "Content-Type: application/json" -H "Accept: application/json" -H "X-Auth-Token:$token" -d '{"port": {"network_id": "$networkId", "tenant_id": "$tenantId", "fixed_ips": [{"ip_address": "$privateIP"}], "name": "$portTag","admin_state_up": true}}' | $tailPath -n 1`);

        if(exists($tempOutput->{'NeutronError'})) {
                my $errorMsg = $tempOutput->{'NeutronError'}{'message'};
                VCSAG_LOG_MSG("E", "Failed to create neutron port due to error $errorMsg",20041,"$errorMsg");
                return 0;
        }

	return 1;
}


# Delete a neutron port.
sub deletePort {
        my $token = shift;
        my $neutronAdminUrl = shift;
	my $portId = shift;

        my $tempOutput =  `$curlPath -s -g -i -X DELETE $neutronAdminUrl/ports/$portId.json -H "User-Agent: python-neutronclient" -H "Accept: application/json" -H "X-Auth-Token:$token"`;

}

#Check If IP is part of the given subnet/cidr or not
# return 1 If IP is part of the subnet 
# return 0 If IP is not part of the subnet
sub IsIpPartOfSubnet {

	my $address = shift;
    	my $block = shift;

    	my @temp = split(/\//, $block);
    	my $netIP = $temp[0];
    	my $netmask = $temp[1];

	if ($address eq $netIP) {
		return 1;
	}

	if ((VCSNetGetIPVersion($address) == 4) && (VCSNetGetIPVersion($netIP) == 4)) {
	        my @sAddress = split(/\./, $address);
        	$address = $sAddress[0] * 2**24 + $sAddress[1] * 2**16 + $sAddress[2] * 2**8 + $sAddress[3];
	
    		my @sNetIP = split(/\./,$netIP);
    		$netIP = $sNetIP[0] * 2**24 + $sNetIP[1] * 2**16 + $sNetIP[2] * 2**8 + $sNetIP[3];

		if($netmask eq "" ) {
			$netmask = 32;
		}
    		if(($address >> (32-$netmask)) == ($netIP >> (32-$netmask))) {
        	  	  return 1;
    		}
	}

    	return 0;
}


# Update Router with the route entry.
sub updateRouter {
        my $routerId = shift;
        my $baseIP = shift;
        my $overlayIP = shift;
	my $token = shift;
	my $neutronAdminUrl = shift;
        my $action = shift;
	my $myUnqEntry;

	if(VCSNetGetIPVersion($baseIP) == 6 ) {
		$myUnqEntry=$baseIP."/128-".$baseIP;
	}
	else {
		$myUnqEntry=$baseIP."/32-".$baseIP;
	}

	my $staticRoute = $overlayIP."-".$baseIP;

	my $prevUnqEntry;
	my $repeatCount = 0;

	while ($repeatCount<=3) {
		my $UnqEntryCount = 0;
		my @allRoutes = getAllRoutesOfRouter($token,$neutronAdminUrl,$routerId);
		foreach my $routeEntry (@allRoutes){
			if($prevUnqEntry eq $routeEntry) {

				#Previous Unique Entry is present.
				$UnqEntryCount=1;
				$repeatCount = $repeatCount + 1;
				#Waiting for 5 seconds before retry.
				sleep 5;
				last;
			}
				
			my @routes = split("-",$routeEntry);
			my @destination = split("/",@routes[0]);

			if(@destination[0] eq @routes[1]) {

				#Some host unique entry is present.
				$UnqEntryCount=1;
				$repeatCount = $repeatCount + 1;	
				$prevUnqEntry = $routeEntry;
				#Waiting for 5 seconds before retry.
				sleep 5;
				last;	
			}
		}
				
		if($UnqEntryCount == 0 || $repeatCount == 4) {

			if($repeatCount == 4) {
				#Remove the Previous Unique entry of any host,gave enough chance.
				@allRoutes = grep {$_ ne $prevUnqEntry} @allRoutes;	
			}
			#Add my host unique entry in the array of routes.
			push @allRoutes,$myUnqEntry;

			#Update the router with all the routes including my unique entry.
			addStaticRoutesOnRouter($token,$neutronAdminUrl,$routerId,\@allRoutes);	

			#Waiting for 2 seconds before proceeding further(let only one win).
			sleep 2;
			@allRoutes = getAllRoutesOfRouter($token,$neutronAdminUrl,$routerId);
			my $UnqEntryFlag= 0;
			foreach my $routeEntry (@allRoutes){
				if($routeEntry eq $myUnqEntry) {

					#My Unique is present,I can proceed further to update the router.
					$UnqEntryFlag=1;
					last;
				}
			}
			if($UnqEntryFlag == 0) {
				#My host Unique entry is removed.I need to come again from the start.
				$repeatCount = 0;
				$prevUnqEntry = "";
			}
			else {
				#Remove my unique entry from the routes.
				@allRoutes = grep {$_ ne $myUnqEntry} @allRoutes;
				if($action eq "Add") {
					#Add the static route into the array.
					push @allRoutes,$staticRoute;
					#Update the router with all the routes.
					addStaticRoutesOnRouter($token,$neutronAdminUrl,$routerId,\@allRoutes);
				}
				elsif($action eq "Remove") {
					#Remove the route from the array.
					@allRoutes = grep {$_ ne $staticRoute} @allRoutes;
					#Update the router with the all routes.
					addStaticRoutesOnRouter($token,$neutronAdminUrl,$routerId,\@allRoutes);
				}
				else {
					#Remove the stale overlayIP from the routes.
					my @newRoutes;
                        	 	foreach my $route (@allRoutes) {
                                		if($route !~ /^$overlayIP/){
                                	        	push @newRoutes,$route;
                               			}	
                       			 }
					#Update the router with all the routes.
                        		addStaticRoutesOnRouter($token,$neutronAdminUrl,$routerId,\@newRoutes);
				}
				#come out of the while loop.
				$repeatCount = 4;
			}
		}
	}
}


sub get_decrypt_token{
        my $enc_pword = shift;
        my  $token = "";

        $enc_pword =~s/^\s+//;
        $enc_pword =~s/\s+$//;
        if ($enc_pword) {
                $token = `$HADECRYPT $enc_pword`;  
        }
        return $token; 
}

1;

